--[[--------------------------------------------------------------------
	GridStatusParty
	Adds Grid statuses for group leader, raid assistant, and master looter.
	Authors: Byron Shelden (Dashkal), Pastamancer, Phanx
	URL: http://www.wowace.com/addons/grid-status-party/
	This file is covered under the BSD-2.0 license. See LICENSE.TXT for details
----------------------------------------------------------------------]]

local _, ns = ...
local GridRoster = Grid:GetModule("GridRoster")
local L = setmetatable(ns.L, { __index = Grid.L })

local GridStatusParty = Grid:GetModule("GridStatus"):NewModule("GridStatusParty")
GridStatusParty.menuName = L["Party Flags"]
GridStatusParty.defaultDB = {
	partyLeader = {
		enable = true,
		color = { r = 0.46, g = 0.78, b = 1, a = 1, ignore = true },
		icon = "Interface\\GroupFrame\\UI-Group-LeaderIcon",
		text = L["PL"],
		priority = 15,
		hideInCombat = true,
	},
	raidLeader = {
		enable = true,
		color = { r = 1, g = 0.28, b = 0.04, a = 1, ignore = true },
		icon = "Interface\\GroupFrame\\UI-Group-LeaderIcon",
		text = L["RL"],
		priority = 15,
		hideInCombat = true,
	},
	raidAssistant = {
		enable = true,
		color = { r = 1, g = 0.5, b = 0, a = 1, ignore = true },
		icon = "Interface\\GroupFrame\\UI-Group-AssistantIcon",
		text = L["RA"],
		priority = 15,
		hideInCombat = true,
	},
	masterLooter = {
		enable = true,
		color = { r = 1, g = 1, b = 0, a = 1, ignore = true },
		icon = "Interface\\GroupFrame\\UI-Group-MasterLooter",
		text = L["ML"],
		priority = 20,
		hideInCombat = true,
	},
}

local function GetOptionsForStatus(status, order)
	local status_name = status
	return {
		text = {
			name = L["Text"],
			desc = L["Text to display for this status on text-based indicators."],
			order = 50,
			type = "input",
			get = function()
				return GridStatusParty.db.profile[status_name].text
			end,
			set = function(_, v)
				GridStatusParty.db.profile[status_name].text = v
				GridStatusParty:UpdateAll()
			end,
		},
		hideInCombat = {
			name = L["Hide in combat"],
			desc = L["Hide this status while you are in combat."],
			type = "toggle",
			get = function()
				return GridStatusParty.db.profile[status_name].hideInCombat
			end,
			set = function(_, v)
				GridStatusParty.db.profile[status_name].hideInCombat = v
				GridStatusParty:UpdateAll()
			end,
		},
	}
end

local numEnabled = 0

function GridStatusParty:PostInitialize()
	self:RegisterStatus("partyLeader", PARTY_LEADER, GetOptionsForStatus("partyLeader"))
	self:RegisterStatus("raidLeader", RAID_LEADER, GetOptionsForStatus("raidLeader"))
	self:RegisterStatus("raidAssistant", RAID_ASSISTANT, GetOptionsForStatus("raidAssistant"))
	self:RegisterStatus("masterLooter", MASTER_LOOTER, GetOptionsForStatus("masterLooter"))

	local opts = Grid.options.args.GridStatus.args.GridStatusParty.args
	opts.partyLeader.order = 1
	opts.raidLeader.order = 2
	opts.raidAssistant.order = 3
	opts.masterLooter.order = 4
end

function GridStatusParty:OnStatusEnable(status)
	self:Debug("OnStatusEnable", status)
	numEnabled = numEnabled + 1
	if numEnabled > 1 then return end
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "UpdateAll")
	self:RegisterEvent("PARTY_LEADER_CHANGED", "UpdateAll")
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED", "UpdateAll")
	self:RegisterMessage("Grid_EnteringCombat", "UpdateAll")
	self:RegisterMessage("Grid_LeavingCombat", "UpdateAll")
end

function GridStatusParty:OnStatusDisable(status)
	self:Debug("OnStatusDisable", status)
	numEnabled = numEnabled - 1
	if numEnabled > 0 then return end
	self:UnregisterEvent("GROUP_ROSTER_UPDATE")
	self:UnregisterEvent("PARTY_LEADER_CHANGED")
	self:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED")
	self:UnregisterMessage("Grid_EnteringCombat")
	self:UnregisterMessage("Grid_LeavingCombat")
end

function GridStatusParty:UpdateAll()
	if IsInRaid() then
		self:UpdateRaid()
	elseif IsInGroup() then
		self:UpdateParty()
	else
		self.core:SendStatusLostAllUnits("partyLeader")
		self.core:SendStatusLostAllUnits("raidLeader")
		self.core:SendStatusLostAllUnits("raidAssistant")
		self.core:SendStatusLostAllUnits("masterLooter")
	end
end

function GridStatusParty:SendStatus(guid, status, gained)
	local db = self.db.profile[status]
	if gained and db.enable and (not db.hideInCombat or not Grid.inCombat) then
		self.core:SendStatusGained(guid, status,
			db.priority,
			nil,
			db.color,
			db.text,
			nil,
			nil,
			db.icon
		)
	else
		self.core:SendStatusLost(guid, status)
	end
end

function GridStatusParty:UpdateParty()
	local _, looterID = GetLootMethod()
	for i = 0, GetNumGroupMembers() - 1 do
		local unit = i > 0 and "party"..i or "player"
		local guid = UnitGUID(unit)
		self:SendStatus(guid, "partyLeader", UnitIsGroupLeader(unit))
		self:SendStatus(guid, "raidLeader", false)
		self:SendStatus(guid, "raidAssistant", false)
		self:SendStatus(guid, "masterLooter", i == looterID)
	end
end

function GridStatusParty:UpdateRaid()
	for i = 1, GetNumGroupMembers() do
		local unit = "raid"..i
		local guid = UnitGUID(unit)
		local _, rank, _, _, _, _, _, _, _, _, isLooter = GetRaidRosterInfo(i)
		self:SendStatus(guid, "partyLeader", false)
		self:SendStatus(guid, "raidLeader", rank == 2)
		self:SendStatus(guid, "raidAssistant", rank == 1)
		self:SendStatus(guid, "masterLooter", isLooter)
	end
end